@@ -31,6 +31,7 @@
  */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <gtk/gtk.h>
 #include <libgimp/gimp.h>
 #include "gimppluginwindow.h"
@@ -268,7 +269,6 @@ blur_dialog()
   GDrawable *drawable;
   gchar buffer[255];
   GimpPreview *preview;
-  GimpPreviewEvent *preview_event;
 
   argc = 1;
   argv = g_new(gchar *, 1);
@@ -393,7 +393,7 @@ blur_dialog()
   gtk_widget_show (vbox);
   gtk_widget_show (pdata->window);
 
-  /*gimp_tile_cache_ntiles (81); /* FIXME */
+  gimp_tile_cache_ntiles (9);  /*  FIXME */
 
   gimp_preview_update (GIMP_PREVIEW (preview));
 
@@ -452,9 +452,8 @@ blur_idle (gpointer data)
   int prev_y, next_y;
   guchar *prev_row, *dest_row, *next_row;
   guchar *curr_row;
-  int pos;
-  int i;
   gdouble scale;
+  gint scaleint;
 
   info = (BlurIdleInfo*) data;
   blur_vals = info->blur_vals;
@@ -484,58 +483,60 @@ blur_idle (gpointer data)
 
       memcpy (src, info->scaled_data, 
 	      sizeof (guchar) * info->preview->width * info->preview->height * 4);
+
+      /* this next one should be nuked */
+      memcpy (dest, info->scaled_data, 
+	      sizeof (guchar) * info->preview->width * info->preview->height * 4);
   
+/*
       printf("\nrandom percent %f\n", blur_vals->blur_pct);
       printf("repeat count %f\n", blur_vals->blur_rcount);
       printf("seed type %d\n", blur_vals->seed_type);
       printf("blur seed %d\n", blur_vals->blur_seed);
+*/
     }
 
 
+/*
   printf("[%d]\n", count);
+*/
 
   scale = info->scale;
-  
+  scaleint = (int) scale;
+
   for (y = 0; y < info->preview->height; y++)
     {
-      prev_y = MAX ((y - scale), 0);
-      next_y = MIN ((y + scale), info->preview->height);
-      
+      prev_y = MAX ((y - scaleint), 0);
+      next_y = MIN ((y + scaleint), info->preview->height - 1);
+
       prev_row = &(src[prev_y * info->preview->width * 4]);
       curr_row = &(src[y * info->preview->width * 4]);
       next_row = &(src[next_y * info->preview->width * 4]);
       dest_row = &(dest[y * info->preview->width * 4]);
-      
+
       for (x = 0; x < info->preview->width * 4; x++)
-	{
-	  prev_x = x - (int)scale * 4;
-	  next_x = x + (int)scale * 4;
+        {
+          prev_x = x - scaleint * 4;
+          next_x = x + scaleint * 4;
 	  
 	  if (prev_x < 0)
-	    prev_x += (int)scale * 4; /* incorrect */
+	    prev_x += scaleint * 4;
 	  
-	  if (next_x > info->preview->height)
-	    next_x -= (int)scale * 4; /* incorrect */
+	  if (next_x >= info->preview->width * 4)
+	    next_x -= scaleint * 4;
 	  
 	  dest_row[x] = (prev_row[prev_x] + prev_row[x] + prev_row[next_x]
 			 + curr_row[prev_x] + curr_row[x] + curr_row[next_x]
 			 + next_row[prev_x] + next_row[x] + next_row[next_x]) / 9;
 	  
-	}
+        }
     }
   
   memcpy (src, dest,
-	  sizeof (guchar) * info->preview->width * info->preview->height * 4);
+          sizeof (guchar) * info->preview->width * info->preview->height * 4);
 
-  for (row = 0; row < info->preview->height; row++)
-    {
-      gimp_preview_draw_row (info->preview, row, 
-			     &(dest[row * info->preview->width * 4]));
-    }
-  
-  gimp_preview_force_redraw (info->preview);
   gtk_progress_bar_update (GTK_PROGRESS_BAR (info->preview->progress_bar), 
-			   (count + 1) / blur_vals->blur_rcount);
+	                   (count + 1) / blur_vals->blur_rcount);
 
   count++;
 
@@ -543,9 +544,21 @@ blur_idle (gpointer data)
     return TRUE;
   else
     {
+      gimp_preview_force_redraw (info->preview);
+      gtk_progress_bar_update (GTK_PROGRESS_BAR (info->preview->progress_bar), 
+	                       1.0);
+
+      for (row = 0; row < info->preview->height; row++)
+        {
+          gimp_preview_draw_row (info->preview, RGBA_IMAGE, row,
+			         &(dest[row * info->preview->width * 4]));
+        }
+      gimp_preview_force_redraw (info->preview);
+
       /* Should we force a redraw of progress bar before we reset it? */
       gtk_progress_bar_update (GTK_PROGRESS_BAR (info->preview->progress_bar),
 			       0.0);
+      gimp_preview_force_redraw (info->preview);
       return FALSE;
     }
 }
